﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Markup;
using System.Reflection;
using System.Windows.Data;

namespace System.Windows.Controls.Extend
{
    public class PivotDataGrid : DataGrid
    {
        // tempdata used to make a new pivoted collection and bind it
        // to the datagrid.
        private ObservableCollection<PivotDataGridItem> tempdata;
        // columnHeaderValues used temporarily to store the all the data
        // which used to be the new headers.
        private List<object> columnHeaderValues;
        // rowdata used temporarily to store the other data which used
        // to be the left rows.
        private Dictionary<object, List<object>> rowdata;

        private IEnumerable _workSource;
        // we hide the base ItemsSource because we need to make them two
        // available at the same time.
        public new IEnumerable ItemsSource
        {
            get
            {
                return _workSource;
            }
            set
            {
                _workSource = value;
                // we pivot the collection here and 
                // create the new collection to bind.
                ApplyWorkDataSource(value);
            }
        }

        // we should know which property will be changed to column header.
        public string HeaderProperty { get; set; }

        public PivotDataGrid()
            : base()
        {

            this.HeadersVisibility = DataGridHeadersVisibility.All;
            this.LoadingRow += new EventHandler<DataGridRowEventArgs>(PivotGrid_LoadingRow);
        }


        private void ApplyWorkDataSource(IEnumerable source)
        {
            // generate the new source
            this.convertItemsSourceForPivotGrid(source);
            this.AutoGenerateColumns = false;
            int i = 0;
            // generate the new columns according to the new source
            foreach (object obj in columnHeaderValues)
            {

                this.Columns.Add(new DataGridTextColumn()
                {
                    Header = obj.ToString(),
                    Binding = new Binding("Cells[" + i.ToString() + "]")
                });

                i++;
            }
            // generate the new collection
            tempdata = new ObservableCollection<PivotDataGridItem>();
            foreach (object objkey in rowdata.Keys)
            {
                PivotDataGridItem item = new PivotDataGridItem();
                foreach (object obj2 in rowdata[objkey])
                {
                    item.Cells.Add(obj2);
                }
                item.RowHeader = objkey;
                tempdata.Add(item);
            }
            // bind the source to the base datagrid
            base.ItemsSource = tempdata;
        }

        private void convertItemsSourceForPivotGrid(IEnumerable original)
        {
            this.columnHeaderValues = new List<object>();
            this.rowdata = new Dictionary<object, List<object>>();
            // the principle is to use Reflection to get all the public
            // properties which need to be show in the columns.
            // we can extend this part to meet different requirements.
            foreach (object obj in original)
            {
                if (obj == null)
                    continue;

                foreach (PropertyInfo pinfo in obj.GetType().GetProperties())
                {
                    if (pinfo != null)
                    {
                        if (pinfo.Name == this.HeaderProperty)
                        {
                            this.columnHeaderValues.Add(pinfo.GetValue(obj, null));
                        }
                        else
                        {
                            if (!rowdata.ContainsKey(pinfo.Name))
                            {
                                this.rowdata.Add(pinfo.Name, new List<object>());
                            }
                            this.rowdata[pinfo.Name].Add(pinfo.GetValue(obj, null));
                        }
                    }
                }
            }
        }

        private void PivotGrid_LoadingRow(object sender, DataGridRowEventArgs e)
        {
            // set the new row header.
            if (tempdata != null && tempdata.Count >= e.Row.GetIndex() + 1)
                e.Row.Header = tempdata[e.Row.GetIndex()].RowHeader;
        }
    }
}
